/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2010 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

/*
 * BindingSubTypePanel.java
 *
 * Created on August 30, 2006, 7:04 PM
 */

package org.netbeans.modules.xml.wsdl.ui.view;

import java.awt.Component;
import java.awt.event.ActionListener;
import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;

import java.util.Vector;
import javax.swing.AbstractButton;
import javax.swing.ButtonGroup;

import javax.swing.DefaultComboBoxModel;
import javax.swing.JComboBox;
import javax.swing.JList;
import javax.swing.plaf.basic.BasicComboBoxRenderer;
import org.netbeans.modules.xml.wsdl.bindingsupport.template.localized.LocalizedTemplate;
import org.netbeans.modules.xml.wsdl.bindingsupport.template.localized.LocalizedTemplateGroup;
import org.netbeans.modules.xml.wsdl.ui.wsdl.util.DisplayObject;
import org.openide.util.NbBundle;


/**
 *
 * @author  skini
 */
public class BindingSubTypePanel extends javax.swing.JPanel {
    
    private LocalizedTemplateGroup mLtg;
    
    private List<TemplatePanel> mPanels = new ArrayList<TemplatePanel>();
    
    private ActionListener mButtonActionListener;
    private LocalizedTemplate.Mode mMode = null;
    
    private boolean tooManyTemplates = false;
    private JComboBox templateComboBox;
    private static final int TEMPLATE_MAX = 1; 
    
    /** Creates new form BindingSubTypePanel */
    public BindingSubTypePanel(LocalizedTemplateGroup ltg, ActionListener buttonActionListener) {
        this.mLtg = ltg;
        this.mButtonActionListener = buttonActionListener;
        initComponents();
        initGUI();
    }
    
    public BindingSubTypePanel(LocalizedTemplateGroup ltg, ActionListener buttonActionListener, LocalizedTemplate.Mode mode) {
        this.mLtg = ltg;
        this.mButtonActionListener = buttonActionListener;
        this.mMode = mode;
        initComponents();
        initGUI();
    }

    void setPanelEnabled(boolean enabled) {
        Enumeration<AbstractButton> elements = buttonGroup1.getElements();
        while (elements.hasMoreElements()) {
            elements.nextElement().setEnabled(enabled);
        }
        
        if (templateComboBox != null) {
            templateComboBox.setEnabled(enabled);
        }
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc="Generated Code">//GEN-BEGIN:initComponents
    private void initComponents() {

        buttonGroup1 = new javax.swing.ButtonGroup();

        setLayout(new java.awt.GridBagLayout());
    }// </editor-fold>//GEN-END:initComponents
   
    private void initGUI() {
        boolean isDefaultAvailable = false;
        ArrayList<String> actualTemplateNames = new ArrayList<String>();
        LocalizedTemplate[] templates = this.mLtg.getTemplate();
        if (templates.length < TEMPLATE_MAX) {
            for (int i = 0; i < templates.length; i++) {
                LocalizedTemplate template = templates[i];
            if (this.mMode == null ||(LocalizedTemplate.Mode.BOTH.equals(template.getMode())) ||
                    this.mMode.equals(template.getMode()) || template.isSkeleton() ){            
                TemplatePanel panel = new TemplatePanel(template, buttonGroup1);
                panel.getRadioButton().addActionListener(this.mButtonActionListener);
                mPanels.add(panel);
                this.add(panel);
            actualTemplateNames.add(templates[i].getName());
                if (template.getDelegate().isDefault()) {
                    setSelectedTemplateName(templates[i].getName());
                    isDefaultAvailable = true;
                }
            }
            }
        if (!isDefaultAvailable && actualTemplateNames.size() > 0 ) {
            setSelectedTemplateName(actualTemplateNames.get(0));
            }
        } else {
            tooManyTemplates = true;
            Vector<DisplayObject> templateVect = new Vector<DisplayObject>();
            LocalizedTemplate defaultTemplate = null;
            LocalizedTemplate firstTemplate = null;
            for (int i = 0; i < templates.length; i++) {
                LocalizedTemplate template = templates[i];
            if (this.mMode == null ||(LocalizedTemplate.Mode.BOTH.equals(template.getMode())) ||
                    this.mMode.equals(template.getMode()) || template.isSkeleton() ){            

                    //TemplatePanel panel = new TemplatePanel(template, buttonGroup1);
                    //panel.getRadioButton().addActionListener(this.mButtonActionListener);
                    //mPanels.add(panel);
                    //this.add(panel);
                    templateVect.add(new DisplayObject(template.getName(), template));
                    if (template.getDelegate().isDefault()) {
                        defaultTemplate = template;
                        isDefaultAvailable = true;
                    }
                    if ( firstTemplate == null) {
                        firstTemplate = template;
                    }                
                }


            }
            
            DefaultComboBoxModel defaultComboBoxModel = new DefaultComboBoxModel(templateVect);
            templateComboBox = new JComboBox(defaultComboBoxModel);
            templateComboBox.setRenderer(new MyComboBoxRenderer());
            templateComboBox.setName("templateComboBox");
            templateComboBox.getAccessibleContext().setAccessibleName("Available Configuration Templates");
            templateComboBox.getAccessibleContext().setAccessibleDescription(NbBundle.getMessage(BindingSubTypePanel.class, "BindingSubTypePanel.templateComboBox.accessibleDescription"));
            templateComboBox.setToolTipText(NbBundle.getMessage(BindingSubTypePanel.class, "BindingSubTypePanel.templateComboBox.accessibleDescription"));
            if (isDefaultAvailable && firstTemplate != null) {
                if (defaultTemplate == null) {
                    defaultTemplate = firstTemplate;
                }
                templateComboBox.setSelectedItem(new DisplayObject(defaultTemplate.getName(), defaultTemplate));
            }
        
            java.awt.GridBagConstraints gridBagConstraints = new java.awt.GridBagConstraints();
            gridBagConstraints.gridx = 0;
            gridBagConstraints.gridy = 0;
            gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
            gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
            gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
            gridBagConstraints.weightx = 0.5;        
            this.add(templateComboBox, gridBagConstraints);     

            //add listener
            templateComboBox.addActionListener(mButtonActionListener);
            
        }
       
    }
    
    class MyComboBoxRenderer extends BasicComboBoxRenderer {        
        @Override
        public Component getListCellRendererComponent(JList list,
                Object value, int index, boolean isSelected, boolean cellHasFocus) {
            LocalizedTemplate template = (LocalizedTemplate) ((DisplayObject)value).getValue();
            if (isSelected) {
                setBackground(list.getSelectionBackground());
                setForeground(list.getSelectionForeground());
                if (-1 < index) {
                    list.setToolTipText(template.getDescription());
                }
            } else {
                setBackground(list.getBackground());
                setForeground(list.getForeground());
            }
            setFont(list.getFont());
            setText((value == null) ? "" : value.toString());            
            return this;
        }
    }
        
    public LocalizedTemplate getBindingSubType() {
        if (!tooManyTemplates) {
            Iterator<TemplatePanel> it = this.mPanels.iterator();
            while (it.hasNext()) {
                TemplatePanel p = it.next();
                if (p.isSelected()) {
                    return p.getTemplate();
                }
            }
        } else {
            if (templateComboBox != null) {
                DisplayObject template = (DisplayObject) templateComboBox.getSelectedItem();
                return (LocalizedTemplate) template.getValue();
            }
        }
        return null;
    }
    
    public void reset(LocalizedTemplateGroup ltg) {
        this.removeAll();
        Iterator<TemplatePanel> it =  mPanels.iterator();
        while(it.hasNext()) {
        	TemplatePanel panel = it.next();
        	panel.getRadioButton().removeActionListener(this.mButtonActionListener);
        	
        }
        mPanels.clear();
        
        if (templateComboBox != null) {
            templateComboBox.removeActionListener(mButtonActionListener);
            templateComboBox = null;
            tooManyTemplates = false;
        }
        
        this.mLtg = ltg;
        initGUI();
        revalidate();
    }
    
    private void setSelectedTemplateName(String templateName) {
        Enumeration<AbstractButton> buttons = buttonGroup1.getElements();
        if (buttons != null) {
            while (buttons.hasMoreElements()) {
                AbstractButton button = buttons.nextElement();
                if (button.getActionCommand().equals(templateName)) {
                    button.setSelected(true);
                }
            }
        }
    }
    
    private ButtonGroup getButtonGroup() {
        return this.buttonGroup1;
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.ButtonGroup buttonGroup1;
    // End of variables declaration//GEN-END:variables
    
}
